package edu.uiuc.dsi;

import java.lang.Runnable;
import java.net.Socket;
import java.nio.ByteBuffer;
import java.util.Arrays;
import java.io.IOException;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;


public class TCPSocket implements Runnable, Comparable<TCPSocket>
{
	private Socket objSocket;
	private Core objCore;
	private BufferedInputStream objInputStream;
	private BufferedOutputStream objOutputStream;
	private String sSocketName;
	private boolean bConnectionReady = false;
	
	public TCPSocket()
	{
		
	}
	
	public TCPSocket(Core core, String selfSocketName)
	{
		objCore = core;
		objSocket = null;
		sSocketName = selfSocketName;
	}
	
	public TCPSocket(Core core, Socket socket)
	{
		objCore = core;
		objSocket = socket;
		
		try
		{
			objInputStream = new BufferedInputStream(socket.getInputStream());
			objOutputStream = new BufferedOutputStream(socket.getOutputStream());
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}
	
	public void run()
	{
		
		try
		{
			byte[] bBuffer = new byte[1500];
			ByteBuffer objBackBuffer = ByteBuffer.allocate(3000);
			
			while (true)
			{
				int iBytesRead = objInputStream.read(bBuffer, 0, 1500);
				objBackBuffer.put(bBuffer, 0, iBytesRead);

				while (true)
				{
					Packet objPacket = Packet.parsePacket(objBackBuffer.array(), objBackBuffer.position());
					
					if (objPacket.isCompletePacket())
					{
						byte[] remainingBytes = Arrays.copyOfRange(objBackBuffer.array(), objPacket.getFullPacketLength(), objBackBuffer.position());
						objBackBuffer.rewind();
						objBackBuffer.put(remainingBytes);

						System.out.print("[R]-->[" + sSocketName + "]: len=" + iBytesRead + "; ");
						System.out.println("data=" + new String(objPacket.getFullPacketBytes()));
						objCore.deliverPacket(this, objPacket);
					}
					else
						break;
				}
			}
		}
		catch (IOException e)
		{
			System.out.println("Unknown IOException occured in edu.uiuc.dsi.TCPSocket thread!");
			System.out.println("   ... this will cause the thread to exit after a stack trace.");
			System.out.println();
			
			e.printStackTrace();
		}
	}
	
	public void writeToSocket(byte[] data)
	{
		if (objSocket == null)
			System.out.println("Write failed. Write attempted to a self-socket.");
		else
		{
			try { System.out.println("[W]--->[" + sSocketName + "]: len=" + data.length + "; data=" + new String(data)); objOutputStream.write(data); objOutputStream.flush(); }
			catch (Exception e) { e.printStackTrace(); }
		}
	}
	
	public String getSocketName()
	{
		return sSocketName;
	}
	
	public void setSocketName(String socketName)
	{
		System.out.println("edu.uiuc.dsi.TCPSocket.setSocketName(...) has renamed socket to \"" + socketName + "\".");
		sSocketName = socketName;
	}
	
	public boolean isConnectionReady()
	{
		return bConnectionReady;
	}
	
	public void setConnectionReadyState(boolean readyState)
	{
		bConnectionReady = readyState; 
	}

	@Override
	public int compareTo(TCPSocket o)
	{
		return sSocketName.compareTo(o.getSocketName()); 
	}
}
